/*  
 *  Tictactoe.java
 *   This applet has a simple tic-toe game
 *         -   one player plays both sides! 
 *  The applet represents the board as a set of buttons.  The buttons
 *  are initially labelled with a blank string, but the label is changed to X
 *  or O if the player clicks on that button.
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class Tictactoe extends Applet 
			implements ActionListener
{
  Font f = new Font("Dialog", Font.BOLD,18);
  Font f2 = new Font("Dialog", Font.BOLD,14);
  Color lightblue = new Color(153,255,255);
  
  Panel top = new Panel ( ) ;
  Panel board = new Panel ( ) ;
 
  Label  display;
  Button reset;
 
  /*  digits is the array of buttons, digitvalues is an array
   *  which saves the String which labels each button
   */ 
  Button digits [ ] = new Button[9];
  String digitvalues [ ] = new String[9];
  
  // xturn is true if it's X's turn and false if it's O's turn
  boolean xturn = true, 
  // win is true is either X or O has 3 in a row
  win = false;  

  public void init() 
  { 
   setBackground(lightblue);
   // extra arguments to GridLayout put 5 pixels of space in-between areas
    board.setLayout ( new GridLayout ( 3, 3, 5, 5));
    
    // create the buttons, add them to the board, and add ActionListener
    for (int i=0; i<digits.length; i++)
      { digits[i] = new Button( " " );
        digitvalues[i] = " " ; 
        digits[i].setFont(f);
        board.add (digits[i]);
        digits[i].addActionListener (this);
      }

    display = new Label ( "X goes first");
    display.setFont ( f );
    top.add ( display );
    
    reset = new Button("Reset");
    reset.setFont ( f2 );
    reset.addActionListener ( this );
    top.add (reset );
   
    setLayout ( new BorderLayout ( ) );     
    add (top, BorderLayout.NORTH);
    add (board, BorderLayout.CENTER);
  }
  
  public void actionPerformed(ActionEvent e) 
  {
    // if a player clicks a button, first make sure that it's an empty space
    // (labelled blank), and that nobody has won yet
    // then label the button either X or O
      for (int i=0; i<digits.length; i++)
      if (e.getSource ( ) == digits [ i ] )
        { 
           if ( digits [ i ].getLabel ( ).equals ( " " ) && ! win )
           {   if ( xturn ) { digits [ i ].setLabel ( "X"); digitvalues[i] = "X" ; }  
                        else { digits [ i ].setLabel ( "O");  digitvalues[i] = "O"; }
            //  this executes the body of the checkwin method down below
            // win will be true if either X or O has 3 in a row
               win = checkwin ( );
               if ( win ) 
               {  if ( xturn ) { display.setText ( "X wins!" ); }
                          else  { display.setText ( "O wins!" ); }
                }
                else  // not win
                { if ( xturn ) { display.setText ( "O's turn" );  xturn = false; }
                         else  {  display.setText ( "X's turn" );  xturn = true; }
                 }
             }
         }

// if the player clicks the reset button, set all the variables back to the beginning
    if (e.getSource ( ) == reset)
      {  
          for (int i=0; i<digits.length; i++)
           { digits[i].setLabel ( " "); digitvalues[i] = " "; }
          display.setText (" X goes first ");
          xturn = true;
          win = false;
      }
  }
  
/* This is a method that checks the board to see if there is any row,
 * column, or diagonal that has 3 letters in a row, either X or O.  This
 * method is called after every move.  It returns the value true if there 
 * are 3 letters in a row, and false if not.  Note that for each row, column
 * or diagonal, it first checks if all three values are the same, and then
 * checks that the values are not blank.
 * Note that two ampersands is the "and" operator for booleans and
 * two vertical bars is the "or" operator.
 */
  boolean checkwin ( ) 
  { if ( 
        // test first row
       ((digitvalues [ 0 ] == digitvalues [ 1 ]  ) &&
          (digitvalues [ 1 ] == digitvalues [ 2 ]  ) &&
          ! (digitvalues [ 2 ] == " "))
       || 
       // test second row          
       (( digitvalues [ 3 ] == digitvalues [ 4 ] ) &&
          (digitvalues [ 4 ] == digitvalues [ 5 ]) &&
          ! (digitvalues [ 5 ] == " " ))
       ||
       // test third row
       (( digitvalues [ 6 ] ==digitvalues [ 7 ] ) &&
         ( digitvalues [ 7 ] == digitvalues [ 8 ] ) &&
          ! (digitvalues [ 8 ] == " " ))
       ||
       // test first column
       (( digitvalues [ 0 ] == digitvalues [ 3 ] ) &&
          ( digitvalues [ 3 ] == digitvalues [ 6 ] ) &&
          ! (digitvalues [ 6 ] == " " ))
       ||
       // test second column
        (( digitvalues [ 1 ] == digitvalues [ 4 ] ) &&
          ( digitvalues [ 4 ] == digitvalues [ 7 ] ) &&
          ! (digitvalues [ 7 ] ==  " "))
       ||
       // test third column
         (( digitvalues [ 2 ] == digitvalues [ 5 ] ) &&
          (digitvalues [ 5 ] == digitvalues [ 8 ] ) &&
          ! (digitvalues [ 8 ] ==  " "))
       ||
        // test first diagonal
         (( digitvalues [ 0 ] == digitvalues [ 4 ] ) &&
           ( digitvalues [ 4 ] == digitvalues [ 8 ] ) &&
          ! (digitvalues [ 8 ] ==  " "))
       ||
       // test second diagonal
          (( digitvalues [ 2 ] == digitvalues [ 4 ] ) &&
          ( digitvalues [ 4 ] == digitvalues [ 6 ] ) &&
          ! (digitvalues [ 6 ] ==  " "))
          )
        { return true; }
      else
         {  return false; }
    }
}

