/*
 * NPACLogo.java
 * Author: Alvin Leung
 *
 * Parallelogram - helper object to draw the penels. Used by NPAC Logo and
 *                 Banner.
 *
 * Copyright Notices
 *   (C) 1995 - 1996      Northeast Parallel Architectures Center at
 *                        Syracuse University
 */

import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.Color;

class Parallelogram {
   Polygon BlackParallelogram = new Polygon();
   Polygon WhiteParallelogram = new Polygon();

   public Parallelogram(int sx, int sy) {
      double          border_width;
      double          slope;
      double          x_offset;
      double          y_intercept;

      double          width;
      double          height;
      double          skew_y;

      double          delta_y1;
      double          delta_y2;

      width = sx * (1.4 / 8.35);
      skew_y = sy * (0.95 / 5.9);
      height = sy * (1.85 / 5.9);

      border_width = Math.max(sx * 2.5 / 256.0, 1.0);

      slope = -skew_y / width;
      x_offset = border_width / Math.abs(slope);
      x_offset = Math.sqrt(x_offset * x_offset + border_width * border_width);
      y_intercept = Math.abs((width + x_offset) * slope);

      delta_y1 = (slope * border_width) + y_intercept;
      delta_y2 = (slope * (width - border_width)) + y_intercept;

      BlackParallelogram.addPoint(0, (int) skew_y);
      BlackParallelogram.addPoint((int) width, 0);
      BlackParallelogram.addPoint((int) width, (int) height);
      BlackParallelogram.addPoint(0, (int) (height + skew_y));

      WhiteParallelogram.addPoint((int) border_width, (int) delta_y1);
      WhiteParallelogram.addPoint((int) (width - border_width), (int) delta_y2);
      WhiteParallelogram.addPoint((int) (width - border_width),
         (int) (height + skew_y - delta_y1));
      WhiteParallelogram.addPoint((int) border_width,
         (int) (height + skew_y - delta_y2));
   }

   public void fill(Graphics g, int x, int y) {
      g.translate(x, y);

      g.setColor(Color.black);
      g.fillPolygon(BlackParallelogram);

      g.setColor(Color.white);
      g.fillPolygon(WhiteParallelogram);

      g.translate(-x, -y);
   }
}

